<?php

namespace Teknisa\FIN\Modules\ContaPagar\Services;

use Doctrine\ORM\EntityManager;
use Financeiro\Entities\Ratctpagar;
use Teknisa\FIN\Modules\ContaPagar\Factory\RateioContaFactory as RateioContaFactory;

/**
 * Class RateioContaService
 * @package Teknisa\FIN\Modules\ContaPagar\Services
 */
class RateioContaService {

    protected $dtsAjustesBoleto;
    protected $entityManager;

    /**
     * @param EntityManager $entityManager
     * @param RateioContaFactory $rateioContaFactory
     */
    function __construct(EntityManager $entityManager, RateioContaFactory $rateioContaFactory)
    {
        $this->entityManager = $entityManager;
        $this->rateioContaFactory = $rateioContaFactory;
    }

    public function geraRateioPag(
        $nrorg, $cdfilial, $cdfilirece, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $cdtpctpagar, $vrorigpag
    ){
        try{
            // Rateio Conta Pagar
            $ratctpagar = $this->rateioContaFactory->buscaRatCtPagar(
                $nrorg, $nrlanctonf, $cdfilial, $nrseqvenpag, $nrparcelapag, $cdfilirece
            );

            // Verifica Existência Rateio Conta Pagar
            if(!$ratctpagar){

                $rattpctp = $this->buscaRateioPadraoContaPagar($nrorg, $cdtpctpagar, $cdfilial, $cdfilirece);

                // Verifica Existência Rateio Padrão Conta Pagar
                if($rattpctp){
                    $ratctpagar = $this->geraRateioContaPagar(
                        $rattpctp, $cdtpctpagar, $nrorg, $cdfilial, $nrlanctonf, $nrseqvenpag,
                        $nrparcelapag, $cdfilirece, $vrorigpag, $ratctpagar
                        );
                }
            } else {
                $this->atualizaRateioContaPagar($rattpctp, $cdtpctpagar, $nrorg, $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $cdfilirece, $vrorigpag, $ratctpagar);
            }

            return $ratctpagar;
        } catch (\Exception $e) {
            throw $e;
            return null;
        }
    }

    public function buscaRateioPadraoContaPagar($nrorg, $cdtpctpagar, $cdfilial, $cdfilirece){
        // Rateios Padrão Conta Pagar
        $rattpctp = $this->rateioContaFactory->buscaRatTpCtP($nrorg, $cdtpctpagar);
        // Verifica Não Existência Rateio Padrão Conta Pagar
        if(!$rattpctp){
            $centcustfil = $this->rateioContaFactory->buscaCentCustFil($nrorg, $cdfilirece);
            // Verifica Existência Centro de Custo Filial
            if($centcustfil){
                // Insere Rateio Tipo Conta Pagar[Temporário]
                $rattpctp[0] = $this->rateioContaFactory->insereRatTpCtP(
                    $nrorg, $cdfilial, $cdfilirece, $centcustfil->getCdcentcust(), 100
                );
            }
        }
        return $rattpctp;
    }

    public function geraRateioContaPagar(
        $rattpctp, $cdtpctpagar, $nrorg, $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $cdfilirece, $vrorigpag, $ratctpagar
        ){
        $vrtotratcnt  = 0;
        $vrratccctpag = 0;
        $chave        = 0;
        // Percorre Rateios Padrões Conta Pagar
        /** @var Rattpctp $rateio */
        foreach($rattpctp as $chave => $rateio){
            // Verifica Filial de Rateio Conta Pagar
            $cdfiratctpag = $rateio->getCdfilial()?$rateio->getCdfirattpctp():$cdfilirece;
            // Calcula Valor Rateio Conta Pagar
            $vrratccctpag = round((round($vrorigpag, 2) * round($rateio->getVrperattpctp(), 2))/100, 2);
            // Insere Rateio Conta Pagar
            $ratctpagar[$chave] = $this->rateioContaFactory->insereRatCtPagar(
                $nrorg, $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $rateio->getCdcentcust(), $cdfiratctpag, $vrratccctpag
            );
            // Soma Valor Total Rateio Conta Pagar
            $vrtotratcnt = round($vrtotratcnt, 2) + $vrratccctpag;
        }
        // Verifica Existência de Diferença do Valor Conta/Rateio
        if (round($vrtotratcnt, 2) <> round($vrorigpag, 2)){
            // Calcula Valor Ajustado Última Conta Pagar
            $vrratccctpag = round($vrratccctpag, 2) + round($vrorigpag, 2) - round($vrtotratcnt, 2);
            // Atualiza Último Rateio Conta Pagar
            $ratctpagar = $this->rateioContaFactory->atualizaRatCtPagar(
                $ratctpagar[$chave], $vrratccctpag
            );
        }
        return $ratctpagar;
    }

    public function atualizaRateioContaPagar(
        $rattpctp, $cdtpctpagar, $nrorg, $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $cdfilirece, $vrorigpag, $ratctpagar
        ){
        $vrtotratcnt  = 0;
        $vrratccctpag = 0;
        $chave        = 0;
        // Percorre Rateios Conta Pagar
        /** @var Rattpctp $rateio */
        foreach($ratctpagar as $chave => $rateio){
            // Calcula Valor Rateio Conta Pagar
            $vrratccctpag = round((round($vrorigpag, 2) * round($rateio->getVrperattpctp(), 2))/100, 2);
            // Atualiza Rateio Conta Pagar
            $ratctpagar[$chave] = $this->rateioContaFactory->atualizaRatCtPagar(
                $rateio, $vrratccctpag
            );
            // Soma Valor Total Rateio Conta Pagar
            $vrtotratcnt = round($vrtotratcnt, 2) + $vrratccctpag;
        }
        // Verifica Existência de Diferença do Valor Conta/Rateio
        if (round($vrtotratcnt, 2) <> round($vrorigpag, 2)){
            // Calcula Valor Ajustado Última Conta Pagar
            $vrratccctpag = round($vrratccctpag, 2) + round($vrorigpag, 2) - round($vrtotratcnt, 2);
            // Atualiza Último Rateio Conta Pagar
            $ratctpagar = $this->rateioContaFactory->atualizaRatCtPagar(
                $ratctpagar[$chave], $vrratccctpag
            );
        }
    }

    /**
     * Finaliza a Inserção e/ou Atualização do(s) Rateio(s) Conta a Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrseqvenpag
     * @param $nrparcelapag
     * @param $rateios
     * @throws \Exception
     */
    public function finalizaRateioConta(
        $nrorg, $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $rateios
    ){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            // Busca Rateios Conta Pagar
            $ratctpagar = $this->rateioContaFactory->buscaRateiosConta(
                $nrorg, $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag
            );

            // Percorre Rateio(s) Antigo(s) da Conta a Pagar
            foreach($ratctpagar as $rateio)
                // Deleta Rateio Conta Pagar
                $this->entityManager->remove($rateio);
            $this->entityManager->flush();

            // Percorre Rateios da Conta a Pagar
            foreach($rateios as $rateio) {

                // Insere Rateio Conta Pagar
                $ratctpagar = $this->rateioContaFactory->insereRateioConta(
                    $nrorg, $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $rateio['CDFIRATCTPAG'], $rateio['CDCENTCUST'], $rateio['VRRATCCCTPAG']
                );

                $this->entityManager->persist($ratctpagar);
            }

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Deleta Rateio Conta a Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrseqvenpag
     * @param $nrparcelapag
     * @param $cdcentcust
     * @throws \Exception
     */
    public function deletaRateioConta(
        $nrorg, $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $cdcentcust
    ){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {

            // Busca Rateio Conta Pagar
            /** @var Ratctpagar $ratctpagar */
            $ratctpagar = $this->rateioContaFactory->buscaRateioConta(
                $nrorg, $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $cdcentcust
            );

            // Verifica Existência Rateio Conta
            if($ratctpagar) {
                // Guarda Filial/Valor Rateio Conta Pagar
                $cdfiratctpag = $ratctpagar->getCdfiratctpag();
                $vrratccctpag = $ratctpagar->getVrratccctpag();
                

                // Deleta Rateio Conta Pagar
                $this->entityManager->remove($ratctpagar);
                $this->entityManager->flush();

                // Busca Centro de Custo do Maior Valor Rateio Conta Pagar
                $cdcentcust = $this->rateioContaFactory->buscaCentroCustoMaiorValorRateioConta(
                    $nrorg, $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $cdfiratctpag, $vrratccctpag
                );
                

                $cdcentcust = $cdcentcust[0]['CDCENTCUST'];

                // Verifica Exintência Maior Valor Rateio Conta Pagar
                if ($cdcentcust) {
                    // Busca Maior Valor Rateio Conta Pagar
                    /** @var Ratctpagar $ratctpagar */
                    $ratctpagar = $this->rateioContaFactory->buscaRateioConta(
                        $nrorg, $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $cdcentcust
                    );

                    // Atualiza Maior Valor Rateio Conta Pagar
                    $ratctpagar = $this->rateioContaFactory->atualizaMaiorValorRateioConta(
                        $ratctpagar, $vrratccctpag
                    );

                    $this->entityManager->persist($ratctpagar);
                }
            }

            // Registra Rateio Excluído
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

}