<?php

namespace Teknisa\FIN\Modules\ContaReceber\Services;

use Doctrine\ORM\EntityManager;
use Financeiro\Entities\Movfluxo;
use Financeiro\Entities\Ajbolrec;
use Financeiro\Entities\Ajctreceber;
use Financeiro\Entities\Boletreceber;
use Financeiro\Entities\Contareceber;
use Financeiro\Entities\Ratctreceber;
use Geral\Entities\Lanccont;
use Teknisa\FIN\Modules\FluxoCaixa\Services\FluxoDeCaixaService as FluxoDeCaixaService;
use Teknisa\FIN\Services\ContabilizacaoService as ContabilizacaoService;

/**
 * Class AjusteContaService
 * @package Teknisa\FIN\Modules\ContaReceber\Services
 */
class AjusteContaService {

    protected $dtsAjustesBoleto;
    protected $entityManager;

    /**
     * @param EntityManager $entityManager
     * @param FluxoDeCaixaService $fluxoDeCaixaService
     * @param ContabilizacaoService $contabilizacaoService
     */
    function __construct(EntityManager $entityManager, FluxoDeCaixaService $fluxoDeCaixaService, ContabilizacaoService $contabilizacaoService)
    {
        $this->entityManager         = $entityManager;
        $this->fluxoDeCaixaService   = $fluxoDeCaixaService;
        $this->contabilizacaoService = $contabilizacaoService;
    }

    /**
     * Deleta Ajuste Conta Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $cdtpctreceb
     * @param $nrlanbolrec
     * @param $cdfiliemit
     * @param $nrparcelarec
     * @param $nrseqvenrec
     * @param $valorajuste
     * @param $nrlanctonf
     * @param $idajuste
     * @param $vrentmfluxo
     * @param $vrajuste
     * @param $valoratual
     * @param $dtajusterec
     * @param $cdempresa
     * @param $cdclassfina
     * @param $idmovfluxo
     * @param $cdajuctrec
     * @param $idstatrec
     * @throws \Exception
     */
    public function deletaAjusteConta(
        $nrorg, $cdfilial, $cdtpctreceb, $nrlanbolrec, $cdfiliemit, $nrparcelarec, $nrseqvenrec, $valorajuste,
        $nrlanctonf, $idajuste, $vrentmfluxo, $vrajuste, $valoratual, $dtajusterec, $cdempresa,
        $cdclassfina, $idmovfluxo, $cdajuctrec, $idstatrec
    ){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            
            self::deletaAjusteContaReceber(
                $nrorg, $cdfilial, $cdtpctreceb, $nrlanbolrec, $cdfiliemit, $nrparcelarec, $nrseqvenrec, $valorajuste,
                $nrlanctonf, $idajuste, $vrentmfluxo, $vrajuste, $valoratual, $dtajusterec, $cdempresa,
                $cdclassfina, $idmovfluxo, $cdajuctrec, $idstatrec
            );
            
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Deleta Ajuste Conta Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $cdtpctreceb
     * @param $nrlanbolrec
     * @param $cdfiliemit
     * @param $nrparcelarec
     * @param $nrseqvenrec
     * @param $valorajuste
     * @param $nrlanctonf
     * @param $idajuste
     * @param $vrentmfluxo
     * @param $vrajuste
     * @param $valoratual
     * @param $dtajusterec
     * @param $cdempresa
     * @param $cdclassfina
     * @param $idmovfluxo
     * @param $cdajuctrec
     * @param $idstatrec
     * @throws \Exception
     */
    public function deletaAjusteContaReceber(
        $nrorg, $cdfilial, $cdtpctreceb, $nrlanbolrec, $cdfiliemit, $nrparcelarec, $nrseqvenrec, $valorajuste,
        $nrlanctonf, $idajuste, $vrentmfluxo, $vrajuste, $valoratual, $dtajusterec, $cdempresa,
        $cdclassfina, $idmovfluxo, $cdajuctrec, $idstatrec
    ){
        try {
            // Atualiza Fluxo de Caixa
            /** @var Movfluxo $movfluxo */
            $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                $nrorg, $cdempresa, $cdfiliemit, $cdclassfina, $dtajusterec, $vrentmfluxo, $idmovfluxo, 'E'
            );
            $this->entityManager->persist($movfluxo);

            // Busca Conta a Receber
            /** @var $conta Contareceber */
            $conta = Contareceber::findOneByCdfilialAndNrlanctonfAndNrseqvenrecAndNrparcelarecAndNrorg(
                $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $nrorg
            );

            // Calcula Valor Conta
            if ($idajuste == 'A')
                 $valor = $conta->getVratuarec() - $valorajuste;
            else $valor = $conta->getVratuarec() + $valorajuste;

            // Atualiza Valor Conta
            $conta->setVratuarec($valor);
            $this->entityManager->persist($conta);

            // Atualiza Valor Boleto
            /** @var $boleto Boletreceber*/
            $boleto = Boletreceber::findOneByNrorgAndCdfilialAndNrlanbolrec($nrorg, $cdfiliemit, $nrlanbolrec);
            $boleto->setVrbolrec($valor);
            $boleto->setVrfinalbrec($valor);
            $this->entityManager->persist($boleto);

            // Atualiza Valor Rateio
            self::atualizaValorRateio(
                $nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $cdfiliemit, $vrajuste, $valoratual, $idajuste, 'E'
            );
            $this->entityManager->flush();

            // Busca Ajuste
            /** @var $ajctreceber Ajctreceber */
            $ajctreceber = Ajctreceber::findOneByCdfilialAndNrlanctonfAndCdajuctrecAndNrseqvenrecAndNrparcelarecAndNrorg(
                $cdfilial, $nrlanctonf, $cdajuctrec, $nrseqvenrec, $nrparcelarec, $nrorg
            );

            // Busca Lançamento Contabil
            /** @var $lanccont Lanccont */
            $lanccont = Lanccont::findByNrlanccontAndCdempresaAndNrorg(
                $ajctreceber->getNrlctbajrec(), $cdempresa, $nrorg
            );

            // Deleta Lançamentos Contabil
            if ($lanccont)
                foreach ($lanccont as $lanc)
                    $this->entityManager->remove($lanc);

            // Deleta Ajuste
            $this->entityManager->remove($ajctreceber);
            $this->entityManager->flush();

            // Contabiliza Conta
            $this->contabilizacaoService->contabilizaRecebimento($cdfilial, $cdempresa, $nrlanctonf, $nrparcelarec, $nrseqvenrec, $idstatrec, $cdtpctreceb);

            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Atualiza Valor Rateio Conta Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrseqvenrec
     * @param $nrparcelarec
     * @param $cdfiratctrec
     * @param $vrajuste
     * @param $valorAtual
     * @param $idajuste
     * @param $tipoOperacao
     * @throws \Exception
     */
    public function atualizaValorRateio(
        $nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $cdfiratctrec, $vrajuste, $valorAtual, $idajuste, $tipoOperacao
    ){
        try {
            // Busca Rateios da Conta
            /** @var $rateios Ratctreceber */
            $rateios = Ratctreceber::findByCdfilialAndNrlanctonfAndNrseqvenrecAndNrparcelarecAndCdfiratctrecAndNrorg(
                $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $cdfiratctrec, $nrorg
            );

            // Verifica Existência Rateios
            if ($rateios) {

                // Variáveis Inicialização
                $valor   = 0;
                $somarat = 0;

                // Percorre Todos Rateios
                /** @var $rateio Ratctreceber */
                foreach ($rateios as $rateio) {
                    
                    // Calcula Valor Rateio
                    $valorPercentual = ($rateio->getVrratccctrec() * 100) / $valorAtual;
                    if ($tipoOperacao == 'I') {
                        if ($idajuste == 'A') {
                            $valor = $vrajuste * ($valorPercentual / 100);
                            $valor = $rateio->getVrratccctrec() + $valor;
                        } else {
                            $valor = $vrajuste * ($valorPercentual / 100);
                            $valor = $rateio->getVrratccctrec() - $valor;
                        }
                    } else {
                        if ($idajuste == 'A') {
                            $valor = $vrajuste * ($valorPercentual / 100);
                            $valor = $rateio->getVrratccctrec() - $valor;
                        } else {
                            $valor = $vrajuste * ($valorPercentual / 100);
                            $valor = $rateio->getVrratccctrec() + $valor;
                        }
                    }
                    $somarat += round($valor, 2);

                    // Atualiza Valor Rateio
                    $rateio->setVrratccctrec($valor);
                    $this->entityManager->persist($rateio);
                }
                
                // Verifica Diferença Valor Rateio
                if ($tipoOperacao != 'I' && $idajuste == 'A' or $tipoOperacao == 'I' && $idajuste != 'A')
                     $valorAjustado = $vrajuste * -1;
                else $valorAjustado = $vrajuste;
                $diferenca = round($valorAtual + $valorAjustado - $somarat, 2);

                // Atualiza Diferença Valor Rateio
                if ($diferenca != 0) {
                    $rateio = $rateios[count($rateios) - 1];
                    $rateio->setVrratccctrec($valor + $diferenca);
                    $this->entityManager->persist($rateio);
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

} 